if (type(AzOptionsFactory) == "table") and (AzOptionsFactory.vers >= 1) then
	return;
end

AzOptionsFactory = { vers = 1, objectCount = {} };

local _G = getfenv(0);

--------------------------------------------------------------------------------------------------------
--                                          Helper Functions                                          --
--------------------------------------------------------------------------------------------------------

-- Converts string colors to RGBA
local function HexStringToRGBA(self,string)
	local a, r, g, b = string:match("^|c(..)(..)(..)(..)");
	return format("%d","0x"..r) / 255, format("%d","0x"..g) / 255, format("%d","0x"..b) / 255, format("%d","0x"..a) / 255;
end

-- Generate Unique Object Name
local function GenerateObjectName(type)
	AzOptionsFactory.objectCount[type] = (AzOptionsFactory.objectCount[type] or 0) + 1;
	return "AzOptionsFactory"..type..AzOptionsFactory.objectCount[type];
end

--------------------------------------------------------------------------------------------------------
--                                            Slider Frame                                            --
--------------------------------------------------------------------------------------------------------

-- EditBox Enter Pressed
local function SliderEdit_OnEnterPressed(self)
	self:GetParent().slider:SetValue(self:GetNumber());
end

-- Slider Value Changed
local function Slider_OnValueChanged(self)
	local parent = self:GetParent();
	parent.edit:SetNumber(self:GetValue());
	parent.factory:ChangeSettingFunc(parent.option.var,self:GetValue());
end

-- OnMouseWheel
local function Slider_OnMouseWheel(self)
	self:SetValue(self:GetValue() + self:GetParent().option.step * arg1);
end

-- New Slider
local function Factory_Slider(self)
	local f = CreateFrame("Frame",nil,self.owner);
	f:SetWidth(292);
	f:SetHeight(32);

	f.edit = CreateFrame("EditBox",GenerateObjectName("EditBox"),f,"InputBoxTemplate");
	f.edit:SetWidth(45);
	f.edit:SetHeight(21);
	f.edit:SetPoint("BOTTOMLEFT");
	f.edit:SetScript("OnEnterPressed",SliderEdit_OnEnterPressed);
	f.edit:SetAutoFocus(nil);
	f.edit:SetMaxLetters(4);

	local sliderName = GenerateObjectName("Slider");
	f.slider = CreateFrame("Slider",sliderName,f,"OptionsSliderTemplate");
	f.slider:SetPoint("TOPLEFT",f.edit,"TOPRIGHT",5,-3);
	f.slider:SetPoint("BOTTOMRIGHT",0,-2);
	f.slider:SetScript("OnValueChanged",Slider_OnValueChanged);
	f.slider:SetScript("OnMouseWheel",Slider_OnMouseWheel);
	f.slider:EnableMouseWheel(1);

	f.text = _G[sliderName.."Text"];
	f.text:SetTextColor(1.0,0.82,0);
	f.low = _G[sliderName.."Low"];
	f.low:ClearAllPoints();
	f.low:SetPoint("BOTTOMLEFT",f.slider,"TOPLEFT",0,0);
	f.high = _G[sliderName.."High"];
	f.high:ClearAllPoints();
	f.high:SetPoint("BOTTOMRIGHT",f.slider,"TOPRIGHT",0,0);

	f.factory = self;
	return f;
end

--------------------------------------------------------------------------------------------------------
--                                           Check Buttons                                            --
--------------------------------------------------------------------------------------------------------

local function CheckButton_OnEnter(self)
	self.text:SetTextColor(1,1,1);
	if (self.option.tip) then
		GameTooltip:SetOwner(self,"ANCHOR_RIGHT");
		GameTooltip:AddLine(self.option.label,1,1,1);
		GameTooltip:AddLine(self.option.tip,nil,nil,nil,1);
		GameTooltip:Show();
	end
end

local function CheckButton_OnLeave(self)
	self.text:SetTextColor(1,0.82,0);
	GameTooltip:Hide();
end

local function CheckButton_OnClick(self)
	self.factory:ChangeSettingFunc(self.option.var,self:GetChecked() == 1);
end

-- New CheckButton
local function Factory_Check(self)
	local f = CreateFrame("CheckButton",nil,self.owner,"OptionsSmallCheckButtonTemplate");
	f:SetWidth(26);
	f:SetHeight(26);
	f:SetScript("OnEnter",CheckButton_OnEnter);
	f:SetScript("OnClick",CheckButton_OnClick);
	f:SetScript("OnLeave",CheckButton_OnLeave);

	f.text = select(6,f:GetRegions());

	f.factory = self;
	return f;
end

--------------------------------------------------------------------------------------------------------
--                                           Color Buttons                                            --
--------------------------------------------------------------------------------------------------------

-- Color Picker Function
local function ColorButton_ColorPickerFunc(prevVal,...)
	local r, g, b, a;
	if (prevVal) then
		r, g, b, a  = unpack(prevVal);
	else
		r, g, b = ColorPickerFrame:GetColorRGB();
		a = (1 - OpacitySliderFrame:GetValue());
	end
	if (ColorPickerFrame.frame.option == ColorPickerFrame.option) then
		ColorPickerFrame.frame.texture:SetVertexColor(r,g,b,a);
	end
	if (ColorPickerFrame.option.subType == 2) then
		ColorPickerFrame.frame.factory:ChangeSettingFunc(ColorPickerFrame.option.var,format("|c%.2x%.2x%.2x%.2x",a * 255,r * 255,g * 255,b * 255));
	else
		ColorPickerFrame.frame.factory:ChangeSettingFunc(ColorPickerFrame.option.var,{ r, g, b, a });
	end
end

-- OnClick
local function ColorButton_OnClick(self,button)
	local r, g, b, a;
	if (self.option.subType == 2) then
		r, g, b, a = self.owner:HexStringToRGBA(self.factory.cfg[self.option.var]);
	else
		r, g, b, a = unpack(self.factory.cfg[self.option.var]);
	end

	ColorPickerFrame.frame = self;
	ColorPickerFrame.option = self.option;
	ColorPickerFrame.func = ColorButton_ColorPickerFunc;
	ColorPickerFrame.cancelFunc = ColorButton_ColorPickerFunc;
	ColorPickerFrame.opacityFunc = ColorButton_ColorPickerFunc;
	ColorPickerFrame.hasOpacity = true;
	ColorPickerFrame.opacity = (1 - a);
	ColorPickerFrame.previousValues = { r, g, b, a };

	OpacitySliderFrame:SetValue(1 - a);
	ColorPickerFrame:SetColorRGB(r,g,b);
	ColorPickerFrame:Show();
end

-- OnEnter
local function ColorButton_OnEnter(self)
	self.text:SetTextColor(1,1,1);
	self.border:SetVertexColor(1,1,0);
	if (self.option.tip) then
		GameTooltip:SetOwner(self,"ANCHOR_RIGHT");
		GameTooltip:AddLine(self.option.label,1,1,1);
		GameTooltip:AddLine(self.option.tip,nil,nil,nil,1);
		GameTooltip:Show();
	end
end

-- OnLeave
local function ColorButton_OnLeave(self)
	self.text:SetTextColor(1,0.82,0);
	self.border:SetVertexColor(1,1,1);
	GameTooltip:Hide();
end

-- New ColorButton
local function Factory_Color(self)
	local f = CreateFrame("Button",nil,self.owner);
	f:SetWidth(18);
	f:SetHeight(18);
	f:SetScript("OnEnter",ColorButton_OnEnter);
	f:SetScript("OnLeave",ColorButton_OnLeave)
	f:SetScript("OnClick",ColorButton_OnClick);

	f.texture = f:CreateTexture();
	f.texture:SetPoint("TOPLEFT",-1,1);
	f.texture:SetPoint("BOTTOMRIGHT",1,-1);
	f.texture:SetTexture("Interface\\ChatFrame\\ChatFrameColorSwatch");
	f:SetNormalTexture(f.texture);

	f.border = f:CreateTexture(nil,"BACKGROUND");
	f.border:SetPoint("TOPLEFT");
	f.border:SetPoint("BOTTOMRIGHT");
	f.border:SetTexture(1,1,1,1);

	f.text = f:CreateFontString(nil,"ARTWORK","GameFontNormal");
	f.text:SetPoint("LEFT",f,"RIGHT",4,1);

	f.factory = self;
	return f;
end

--------------------------------------------------------------------------------------------------------
--                                           DropDown Frame                                           --
--------------------------------------------------------------------------------------------------------

-- New DropDown
local function Factory_DropDown(self)
	local f = AzDropDown.CreateDropDown(self.owner,180,true,nil,nil);
	f.text = f:CreateFontString(nil,"ARTWORK","GameFontNormalSmall");
	f.text:SetPoint("LEFT",-302 + f:GetWidth(),0);
	f.factory = self;
	return f;
end

--------------------------------------------------------------------------------------------------------
--                                             Text Edit                                              --
--------------------------------------------------------------------------------------------------------

-- OnTextChange
local function TextEdit_OnTextChanged(self)
	self.factory:ChangeSettingFunc(self.option.var,self:GetText():gsub("||","|"));
end

-- New Text Edit
local function Factory_Text(self)
	local f = CreateFrame("EditBox",GenerateObjectName("EditBox"),self.owner,"InputBoxTemplate");
	f:SetWidth(170);
	f:SetHeight(21);
	f:SetScript("OnTextChanged",TextEdit_OnTextChanged);
	f:SetAutoFocus(nil);

	f.text = f:CreateFontString(nil,"ARTWORK","GameFontNormalSmall");
	f.text:SetPoint("LEFT",-128,1);

	f.factory = self;
	return f;
end

--------------------------------------------------------------------------------------------------------
--                                       Create Factory Instance                                      --
--------------------------------------------------------------------------------------------------------

-- Creates New Factory Instance
function AzOptionsFactory:New(owner,cfg,ChangeSettingFunc)
	return {
		owner = owner,
		cfg = cfg,

		HexStringToRGBA = HexStringToRGBA,
		ChangeSettingFunc = ChangeSettingFunc,

		Slider = Factory_Slider,
		Check = Factory_Check,
		Color = Factory_Color,
		DropDown = Factory_DropDown,
		Text = Factory_Text,
	};
end